/*******************************************************************************

    TINY TERMOMETER2

    C/ PIC12F675 E LEDS EM CONFIGURAO DE 7 SEGUIMENTOS (CHARLIPLEXADO)

    SENSOR DE TEMPERATURA DIGITAL -  DB18B20 (FAIXA -55 A 125C MAS LIMITADO A
    99C POR USAR APENAS 2 DGITOS)

    AUTOR: CLAUDIO LARIOS

    DATA:30/04/2025

    APLICAO: TERMMETRO ECONMICO DE USO DOMESTICO
    (MEDIR TEMPERATURA DO AMBIENTE)

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT

//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  ansel  = 0x9f
#byte  cmcon  = 0x19

//pinos do PIC
#bit p7     = 0x5.0  //pino 7
#bit p6     = 0x5.4  //pino 6
#bit p5     = 0x5.2  //pino 5
#bit p4     = 0x5.3  //pino 4 (entrada apenas)
#bit p3     = 0x5.4  //pino 3
#bit p2     = 0x5.5  //pino 2
#bit dq     = 0x5.5  //pino 2
#bit trisdq = 0X85.5 //sentido do pino (in/out)

//==============================================================================

//constantes para converso 7 seguimentos
//                           D1
//                           |C1
//                           || B1
//                           || |A1
//                         XX||X||*
//                   GPIO  76543210
int8 const NUMP7D[] = {  0B00110110,//0
                         0B00010100,//1
                         0B00100110,//2
                         0B00110110,//3
                         0B00010100,//4
                         0B00110010,//5
                         0B00110010,//6
                         0B00010110,//7
                         0B00110110,//8
                         0B00110110,//9

                                       };

                                       //constantes para converso 7 seguimentos
//                           A0
//                           |G1
//                           || F1
//                           || | E1
//                         XX||X|*|
//                   GPIO  76543210
int8 const NUMP6D[] = {  0B00000101,//0
                         0B00000000,//1
                         0B00010001,//2
                         0B00010000,//3
                         0B00010100,//4
                         0B00010100,//5
                         0B00010101,//6
                         0B00000000,//7
                         0B00010101,//8
                         0B00010100,//9

                                       };

                                       //constantes para converso 7 seguimentos
                                       //* significa pino out gnd
                                       //Y=situao do seguimento A0 a ser transportado
//
//                              F0
//                              |D0
//                              ||B0
//                         YXX*X|||
//                   GPIO  76543210
int8 const NUMP3U[] = {  0B10000111,//0
                         0B00000001,//1
                         0B10000011,//2
                         0B10000011,//3
                         0B00000101,//4
                         0B10000110,//5
                         0B10000110,//6
                         0B10000001,//7
                         0B10000111,//8
                         0B10000111,//9

                                       };
                                       //constantes para converso 7 seguimentos
                                       //* significa pino out gnd
                                       //Y=situao do seguimento A0 a ser transportado
//                           C
//                           |G0
//                           ||  E0
//                           ||  |C0
//                           ||X*||
//                   GPIO  76543210
int8 const NUMP5U[] = {  0B00100011,//0
                         0B00100001,//1
                         0B00110010,//2
                         0B00110001,//3
                         0B00110001,//4
                         0B00110001,//5
                         0B00110011,//6
                         0B00100001,//7
                         0B00110011,//8
                         0B00110001,//9

                                       };




//variveis globais
unsigned int1 neg;//flag de temperatura negativa
unsigned int8 o7dd,o6dd,o5du,o3du,o2led;//acumula valores para GPIO
unsigned int8 t7dd,t6dd,t5du,t3du,t2led;//acumula valores para TRISIO
unsigned int8 cnt,dcnt,tmp;//contadores gerais
unsigned int16 valor;//resultado da leitura do DS18B20


/**************************************************************
       ROTINA DE CONVERSO DE HEXADECIMAL PARA DECIMAL
**************************************************************/
//converte para decimal (0 a 99)
unsigned int8 conv_dec(unsigned int8 a){
unsigned int8 dez;
  if(a>99)a=99;//limita a 2 casas
 //-------------------------------------------------------------
 // if(a>54)o2led=1;//acende led vermelho
 // else if(a>35)o2led=3;//acende leds vermelho e verde
 // else {
 //   o2led=2;//acende somente led verde
 // }
 //------------------------------------------------------------
  o2led=2;//temperatura negativa -> acende led verde apenas
  if(neg==0)o2led=1;//temperatura positiva -> acende leds vermelho
 //-------------------------------------------------------------
  t2led=o2led ^ 0b11011111;//p2-bit5 deve permanecer 0 (out)
  dez=0;
  while(a>=10){a=a-10; dez++; }//converte para decimal as dezenas
  dez=dez<<4;
  dez=dez + a;//soma unidades
  return (dez);//retorna dez/un em 8 bits
}
/**************************************************************
       ROTINA DE CONVERSO DECIMAL PARA SETE SEGUIMENTOS
**************************************************************/
//converte para sete seguimentos em modo charlipex
void conv_7ch (unsigned int8 a){
unsigned int8 b,c;
  b=a;
  b=b>>4;//obtem somente dezenas
  o7dd=NUMP7D[b];//etapa 1- pega dgitos que acendero conforme 'b'
  t7dd=o7dd ^ 0b11111110;// p7 - bit0 deve permanecer 0 (out)

  b=a;
  b= b & 0x0f; //limita a unidades
  o3du=NUMP3U[b];//etapa 2- pega dgitos que acendero conforme 'b'
  c=o3du & 0x80;//salva bit 7 com situao do seguimento 'a0'
  t3du=t3du & 0x7f;//apaga bit 7
  t3du=o3du ^ 0b11101111; //p3 -bit4 deve permanecer 0 (out)

  b=a;
  b=b>>4;//obtem somente dezenas
  o6dd=NUMP6D[b];//etapa 3- pega dgitos que acendero conforme 'b'
  //obs. ajustar bit 5 - seguimento 'a0'
  if(c>0)o6dd = o6dd | 0b00100000;//se maior que 0 seta pino 5 (a0 ligado)
  t6dd=o6dd ^0b11111101;//p6 - bit1 deve permanecer 0 (out) ;

  b=a;
  b= b & 0x0f; //limita a unidades
  o5du=NUMP5U[b];//etapa 4- pega dgitos que acendero conforme 'b'
  t5du=o5du ^ 0b11111011; //p5 -bit2 deve permanecer 0 (out)
}
/**************************************************************
       ROTINA DE MULTIPLEXAO DOS LEDS
**************************************************************/
void multiplexa(){

  trisio=0xff;
  gpio=o7dd;
  trisio=t7dd;
  delay_ms(1);
  trisio=0xff;
  gpio=o6dd;
  trisio=t6dd;
  delay_ms(1);
  trisio=0xff;
  gpio=o5du;
  trisio=t5du;
  delay_ms(1);
  trisio=0xff;
  gpio=o3du;
  trisio=t3du;
  delay_ms(1);
  trisio=0xff;
  gpio=o2led;
  trisio=t2led;
  delay_ms(1);
}
/**************************************************************
       ROTINA DE DELAY COM MULTIPLEXAO
**************************************************************/

void delay_mpx750ms(){
unsigned int8 a;
  for(a=0;a<152;a++){
    multiplexa();
  }
}

//==============================================================================
//           ROTINA DE RESET 1W
//==============================================================================
unsigned int1 reset (void){
     unsigned int1 resp;
      dq=0;
      trisdq=0;//sada
      delay_us(480); //delay de reset
      trisdq=1;//alta impedancia
      delay_us(60); //aguarda resposta do ds1820 com '0'
      resp= dq;
      delay_us(420);
      return (resp);
}
//==============================================================================
//           ROTINA DE LEITURA DE 1 BIT
//==============================================================================
unsigned int1 le_bit(){
  unsigned int1 f;
   dq=0;
   trisdq=0; // '0'
   trisdq=1; //hi
   delay_us(15);
   f=dq;  //correto
   return (f);
}
//==============================================================================
//           ROTINA DE ESCRITA DE 1 BIT
//==============================================================================
void write_1w (unsigned int1 bit){
   dq=0;
   trisdq=0; // '0'
   trisdq=bit;// high_impedncia ou zero
   delay_us(120);//delay
   trisdq=1;// high_impedncia
}
//==============================================================================
//           ROTINA DE ENVIO DE COMANDO
//==============================================================================
void comando (unsigned int8 dado){
unsigned int8 i,d;
   for (i=0;i<8;i++) {
      d=dado&0x01;//testa bit 0
      dado=dado>>1;//desloca para direita 1 bit
      write_1w(d); //envia um bit para o barramento
      }
 }
//==============================================================================
//           ROTINA DE LEITURA DE 1 BYTE
//==============================================================================
unsigned int8 le_byte(){
unsigned int8 i,dado=0;
   for (i=0;i<8;i++) {
    shift_right(&dado,1,le_bit());//pega valor do bit
    delay_us(90);//delay
   }
   return (dado);//retorna 8 bits
}
/*******************************************************************************
        FAZ LEITURA DOS SENSORES DS18B20
*******************************************************************************/
void get_18B20(){// resultados em valor
unsigned int8 b;
     trisio=0x3f;
     reset();//reset
     comando(0xcc);//skip comando
     comando(0x44);//converter sensor escolhido
   //  delay_ms(755);//tempo da converso 12 bits
     delay_mpx750ms();
     trisio=0x3f;
     reset();//reset
     comando(0xcc);//skip comando
     comando(0xbe);//comando para leitura
     valor=le_byte();//le byte lsb
     b=le_byte();//le byte msb
     valor=valor + ((int16) b<<8);//concatena em 16 bits
}
/*******************************************************************************
       SUPRIME LEITURA DE 85 NA PRIMEIRA LEITURA APS O RESET DO DS18B20
*******************************************************************************/
void suprime_85(){
   reset();//reset
   comando(0xcc);//skip comando
   comando(0x44);//converter sensor escolhido
   reset();
}


/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/
  void main(){
   ansel=0;
   cmcon=7;

  for(;;){

   get_18b20();//le temperatura no sensor DS18B20 (pino 7 do PIC) para registrador 'valor'
   neg=0;

   if(valor >= 0x8000) {
     valor= 0xffff - valor;
     valor= valor + 2;
     neg=1;//temperatura negativa
   }
   cnt= valor>>4;



   if(cnt>99)cnt=99;

   dcnt=conv_dec(cnt);
   conv_7ch(dcnt);

   for(tmp=0;tmp<255;tmp++){
     multiplexa();
   }

 }//for(;;)

}//void main()





